;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No More Sprite Tile Limits v1.0.1
;; coded by Edit1754, macro'd by MathOnNapkins
;;
;; To enable this in a level, select sprite header setting 10. It has no effect otherwise.
;;
;; It appears to be incompatible with these: (only when using SP header 10)
;; - Lakitu
;; - Group of 5 eeries
;; - Amazing Flying Hammer Brother (the flying block platform is fine, though)
;; - Boo Buddies Ceiling
;; - Re-Appearing Boo Buddies
;; - Swooper Death Bat Ceiling
;; - Koopa Kid bosses (except for Lemmy and Wendy - the non-mode7 bosses)
;;
;; **IMPORTANT NOTE**
;;   you must go to levels 19B & 1C7 (Bowser Battles)
;;   and change the sprite header setting to 00
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

header
sa1rom

!foundSlot = PickOAMSlot_foundSlot

macro speedup(offset)
		LDA.w $62FD+<offset>	; get Y position of PREVIOUS tile in OAM
		CMP.b #$F0		; F0 means it's free (Y=F0 means it can't be seen)
		BEQ ?notFound		; \  if last isn't free
		LDA.b #<offset>		;  | (and this is), then
		JMP !foundSlot		; /  this is the index
?notFound:
endmacro

macro bulkSpeedup(arg)
		%speedup(<arg>+12)
		%speedup(<arg>+8)
		%speedup(<arg>+4)
		%speedup(<arg>)
endmacro

org $0180D2

SpriteOAMHook:            
		BRA .cutToTheChase	; skip the NOP's
		NOP			; \
		NOP			;  | use NOP
		NOP			;  | to take
		NOP			;  | up space
		NOP			;  | to overwrite
		NOP			;  | old code
		NOP			;  |
		NOP			;  |
		NOP			;  |
		NOP			;  |
		NOP			;  |
		NOP			;  |
		NOP			; /
.cutToTheChase	autoclean JSL PickOAMSlot		; JSL to new code

freecode

CodeStart:

PickOAMSlot:
		LDA $7692		; \  if sprite header
		CMP #$10		;  | setting is not 10,
		BNE .default		; /  use the old code
.notLastSpr	LDA !sprite_status,x		; \ it's not necessary to get an index
		BEQ .return		; / if this sprite doesn't exist
		LDA !sprite_num,x		; \  give yoshi
		CMP #$35		;  | the first
		BEQ .yoshi		; /  two tiles
		BRA SearchAlgorithm	; search for a slot
.foundSlot	STA !sprite_oam_index,x		; set the index
.return		RTL			

.yoshi		LDA #$28		; \ Yoshi always gets
		STA !sprite_oam_index,x		; / first 2 tiles (28,2C)
		RTL

.default	PHX			; \
		TXA			;  | for when not using
		LDX $7692		;  | custom OAM pointer
		CLC			;  | routine, this is
		ADC $07F0B4,x		;  | the original SMW
		TAX			;  | code.
		LDA $07F000,x		;  |
		PLX			;  |
		STA !sprite_oam_index,x		; /
		RTL
    
SearchAlgorithm:
		%bulkSpeedup($F0)	; \
		%bulkSpeedup($E0)	;  | pre-defined
		%bulkSpeedup($D0)	;  | macros with
        %bulkSpeedup($C0)
		%bulkSpeedup($B0)	;  | code for each
		%bulkSpeedup($A0)	;  | individual
		%bulkSpeedup($90)	;  | slot check
		%bulkSpeedup($80)	;  |
		%bulkSpeedup($70)	;  |
		%bulkSpeedup($60)	;  |
		%bulkSpeedup($50)	;  |
		%bulkSpeedup($40)	;  |
		%speedup($3C)		;  |
		%speedup($38)		;  |
		%speedup($34)		; /
		LDA.w $78E2		; \ Yoshi?
		BNE .yoshiExists	; /
		%speedup($30)		; \ More slot
		%speedup($2C)		; / checks
		LDA.b #$28		; \ if none of the above yield which slot
		JMP !foundSlot		; / then use the slot at the beginning
		
.yoshiExists	LDA.b #$30		; \ if none of the above yield which slot,
		JMP !foundSlot		; / then use the slot at the beginning (after Yoshi)